package com.canon.ui.tools.fontCheck;

import java.awt.Font;
import java.io.FileWriter;
import java.io.IOException;
import java.io.UnsupportedEncodingException;
import java.util.LinkedList;

public class FontTestDataCreator {


	class StringDataMap {
		public byte gb18030[];
		public int gb18030i;

		public byte[] utf8;
		public int utf8i;
		public int utf16i;
		public byte[] utf16;
		public String str;
		public byte[] getBytes() {
			return gb18030;
		}
		public void setBytes(byte[] bytes) {
			this.gb18030 = bytes;
		}
		public int getBytesi() {
			return gb18030i;
		}
		public void setBytesi(int bytesi) {
			this.gb18030i = bytesi;
		}
		public byte[] getGb18030() {
			return gb18030;
		}
		public void setGb18030(byte[] gb18030) {
			this.gb18030 = gb18030;
		}
		public byte[] getUtf8() {
			return utf8;
		}
		public void setUtf8(byte[] utf8) {
			this.utf8 = utf8;
		}
		public byte[] getUtf16() {
			return utf16;
		}
		public void setUtf16(byte[] utf16) {
			this.utf16 = utf16;
		}
		public StringDataMap(byte[] bytes) {
			super();
			this.gb18030 = bytes;
	
			try {
				String str = new String(bytes,"GB18030");
				utf8 = str.getBytes("UTF-8");
				utf16 = str.getBytes("UTF-16");
			} catch (UnsupportedEncodingException e) {
				// TODO 自動生成された catch ブロック
				e.printStackTrace();
			}
			
			
			utf8i = getStringCodeAsInt(utf8);
			utf16i = getStringCodeAsInt(utf16);
			
			
		}
		
	}
	public LinkedList<StringDataMap> asciiArray = new LinkedList<StringDataMap>();
	public LinkedList<StringDataMap> gb180302byteCodeList = new LinkedList<StringDataMap>();
	public LinkedList<StringDataMap> gb180304byteCodeList = new LinkedList<StringDataMap>();

	public LinkedList<StringDataMap> gbk1CodeList = new LinkedList<StringDataMap>();
	public LinkedList<StringDataMap> gbk2CodeList = new LinkedList<StringDataMap>();
	public LinkedList<StringDataMap> gbk3CodeList = new LinkedList<StringDataMap>();
	public LinkedList<StringDataMap> gbk4CodeList = new LinkedList<StringDataMap>();
	public LinkedList<StringDataMap> gbk5CodeList = new LinkedList<StringDataMap>();
	public LinkedList<Integer> gbkCodeListIntList = null;

	
	private boolean logmode=true;

	public void setLogmode(boolean b) {logmode = b;}

	private void println(String str) {
		if(logmode) {
			System.out.println(str);
		}
	}


	public void createGBK(LinkedList<StringDataMap> list,int s1,int e1,int s2,int e2) {
		byte byte2Str[] = new byte[2];
		for(int b1 =  s1 ; b1 <= e1;b1++) {
			byte2Str[0] = (byte)b1;
			for(int b2 = s2 ; b2 <= e2;b2++) {

				byte2Str[1] = (byte)b2;
				//			list.add(byte2Str);
				updateLinkedListWithCheck(list,byte2Str);

			}
		}
	}

	/* GBK1 A1-A9 A1-FE*/
	public void createGBK1StringCode() {
		createGBK(gbk1CodeList,0xA1,0xA9,0xA1,0xFE);
	}

	/*GBK2 B0‐F7	A1‐FE*/
	public void createGBK2StringCode() {
		createGBK(gbk2CodeList,0xB0,0xF7,0xA1,0xFE);
	}

	/*GBK3 	81‐A0	(40‐7E、80‐FE)*/
	public void createGBK3StringCode() {
		createGBK(gbk3CodeList,0x81,0xA0,0x40,0x7E);
		createGBK(gbk3CodeList,0x81,0xA0,0x80,0xFE);
	}

	/*GBK4 AA‐FE	(40‐7E、80‐A0)*/
	public void createGBK4StringCode() {
		createGBK(gbk4CodeList,0xAA,0xFE,0x40,0x7E);
		createGBK(gbk4CodeList,0xAA,0xFE,0x80,0xA0);
	}

	/*GBK5 A8‐A9	(40‐7E、80‐A0)*/
	public void createGBK5StringCode() {
		createGBK(gbk5CodeList,0xA8,0xA9,0x40,0x7E);
		createGBK(gbk5CodeList,0xA8,0xA9,0x80,0xA0);
	}

	public void createAsciiCodeList() {
		/*ASCII 0x20-0x7E*/
		asciiArray.clear();

		byte byteStr[] = new byte[1];
		for(byte i=0x20;i<=0x7E;i++) {
			byteStr[0] = i;
			updateLinkedListWithCheck(asciiArray,byteStr);


		}
	}

	private void checkContains(LinkedList<Integer> difflist,LinkedList<Integer> base,LinkedList<Integer> targ) {
		
		base.forEach(ba -> {
			if(!targ.contains(ba)) {
				difflist.add(ba);
				//System.out.println("ba:"+ Integer.toHexString(ba));
			}
			
		});

	}

	public void checkGBKAlltoGB18030_2byte() {





	}

	private int getStringCodeAsInt(byte[] code) {
		
		
		int ret=0;
		int len = code.length;
		int b[] = {0x000000FF,0x0000FFFF,0x00FFFFFF,0xFFFFFFFF};
		int offset;
		for(int i=0;i<len;i++) {
			int temp= 0;
			offset = len-i-1;
			temp|=b[offset] & (code[i] << offset*8);
			
			ret |=temp;
		}

		
		//int ret =  ByteBuffer.wrap(code).getInt();
		return ret;
	}


	public LinkedList<Integer> getStringCodeGBKAsIntegerList() {

		if(gbkCodeListIntList == null) {
			gbkCodeListIntList  = new LinkedList<Integer>();

			gbk1CodeList.stream().forEach(a -> gbkCodeListIntList.add(a.gb18030i));
			gbk2CodeList.stream().forEach(a -> gbkCodeListIntList.add(a.gb18030i));
			gbk3CodeList.stream().forEach(a -> gbkCodeListIntList.add(a.gb18030i));
			gbk4CodeList.stream().forEach(a -> gbkCodeListIntList.add(a.gb18030i));
			gbk5CodeList.stream().forEach(a -> gbkCodeListIntList.add(a.gb18030i));
		}

		return gbkCodeListIntList;
	}
	
	public LinkedList<Integer> getStringCodeByteToInt(LinkedList<byte[]> list) {
		LinkedList<Integer> ret = new LinkedList<Integer>();
		list.stream().forEach( a -> ret.add(getStringCodeAsInt(a)));
		return ret;
	}
	
//	public LinkedList<byte[]> getStringCodeGB18030AsByteLinkedList() {
//		LinkedList<byte[]> ret = new LinkedList<byte[]>();
//
//		asciiArray.stream().forEach( a -> ret.add(a));
//		gb180302byteCodeList.stream().forEach( a -> ret.add(a));
//		gb180304byteCodeList.stream().forEach( a -> ret.add(a));
//
//		ret.stream().sorted();
//
//		return ret;
//
//	}

//	public LinkedList<Integer> getStringCodeGB18030AsIntegerLinkedList() {
//		LinkedList<Integer> ret = new LinkedList<Integer>();
//
//		asciiArray.stream().forEach( a -> ret.add(getStringCodeAsInt(a)));
//		gb180302byteCodeList.stream().forEach( a -> ret.add(getStringCodeAsInt(a)));
//		gb180304byteCodeList.stream().forEach( a -> ret.add(getStringCodeAsInt(a)));
//		ret.stream().sorted();
//
//		return ret;
//
//	}

	private void updateLinkedListWithCheck(LinkedList<StringDataMap> array,byte[] ba) {
		Font font = new Font(Font.SERIF, Font.PLAIN,24);
		try {
			String str = new String(ba,"GB18030");
			char c = str.charAt(0);
			if(font.canDisplay(c)) {
				StringDataMap data=new StringDataMap(ba);
				
			
				array.add(data);
			} else {
				println("GB18030:"+javax.xml.bind.DatatypeConverter.printHexBinary(ba) + " doesn't have glyph");
			}

		} catch (UnsupportedEncodingException e) {
			// TODO 自動生成された catch ブロック
			//e.printStackTrace();
			println("GB18030:"+javax.xml.bind.DatatypeConverter.printHexBinary(ba) + "can not convert to Unicode");
		}
	}

	public void createGB18030StringCode() {

		createAsciiCodeList();

		/*２バイト文字*/
		{
			byte byte2Str[] = new byte[2];

			for(int b1 =  0x81 ; b1 <= 0xFE;b1++) {
				byte2Str[0] = (byte)b1;
				for(int b2 = 0x40 ; b2 <= 0x7E;b2++) {

					byte2Str[1] = (byte)b2;
					
					
					updateLinkedListWithCheck(gb180302byteCodeList,byte2Str);

				}

				for(int b2 = 0x80 ; b2 <= 0xFE;b2++) {

					byte2Str[1] = (byte)b2;

					updateLinkedListWithCheck(gb180302byteCodeList,byte2Str);

				}


			}

		}

		/*４バイト文字*/
		{
			byte byte4Str[] = new byte[4];

			for(int b1 =  0x81 ; b1 <= 0xFE;b1++) {
				byte4Str[0] = (byte)b1;
				for(int b2 =  0x30 ; b2 <= 0x39;b2++) {

					byte4Str[1] = (byte)b2;

					for(int b3 =  0x81 ; b3 <= 0xFE;b3++) {
						byte4Str[2] = (byte)b3;
						for(int b4 =  0x30 ; b4 <= 0x39;b4++) {

							byte4Str[3] = (byte)b4;
							updateLinkedListWithCheck(gb180304byteCodeList,byte4Str);

						}
					}

				}

			}

		}

	}

	public void printString(LinkedList<StringDataMap> list,String encode) {

	}

	
	public void saveFile(LinkedList<StringDataMap> map) {
        try {
            FileWriter fw = new FileWriter("codelist.txt");
            map.stream().sorted();
            for(StringDataMap m :map) {
            String temp;
            temp = "0x" + javax.xml.bind.DatatypeConverter.printHexBinary(m.gb18030);
            String str = new String(m.gb18030,"GB18030");
            temp+=","+javax.xml.bind.DatatypeConverter.printHexBinary(str.getBytes("UTF-8"));
            temp+=","+javax.xml.bind.DatatypeConverter.printHexBinary(str.getBytes("UTF-16"))+"\n";
            fw.write(temp);
            }
            fw.close();
        } catch (IOException ex) {
            ex.printStackTrace();
        }
	}
	public static void main(String[] args) {
		// TODO 自動生成されたメソッド・スタブ
		FontTestDataCreator ftdc = new FontTestDataCreator();

		ftdc.setLogmode(false);
		ftdc.createGB18030StringCode();
		ftdc.createGBK1StringCode();
		ftdc.createGBK2StringCode();
		ftdc.createGBK3StringCode();
		ftdc.createGBK4StringCode();
		ftdc.createGBK5StringCode();
		int gbk234 =  ftdc.gbk4CodeList.size() + ftdc.gbk3CodeList.size() + ftdc.gbk4CodeList.size();
		int gbk15 = ftdc.gbk1CodeList.size() + ftdc.gbk5CodeList.size();
		int gbksum = gbk15 + gbk234;
		ftdc.checkGBKAlltoGB18030_2byte();

		ftdc.saveFile(ftdc.gbk1CodeList);
		System.out.println("GBK1  string num:" + ftdc.gbk1CodeList.size());
		System.out.println("GBK2  string num:" + ftdc.gbk2CodeList.size());
		System.out.println("GBK3  string num:" + ftdc.gbk3CodeList.size());
		System.out.println("GBK4  string num:" + ftdc.gbk4CodeList.size());
		System.out.println("GBK5  string num:" + ftdc.gbk5CodeList.size());

		System.out.println("GBK1/5 sum:"+gbk15+" GBK2-4:"+gbk234+ " GBK All="+gbksum);

		System.out.println("GB18030 1byte string num:" + ftdc.asciiArray.size());
		System.out.println("GB18030 2byte string num:" + ftdc.gb180302byteCodeList.size());
		System.out.println("GB18030 4byte string num:" + ftdc.gb180304byteCodeList.size());


	}

}
